@@ -49,8 +49,7 @@
 
 #include "missing.h"
 #include "alloc.h"
-#include "error.h"
-#include "errno.h"
+#include "fatal.h"
 
 #define DEFAULT_TEXT_DOMAIN	"sudo"
 #include "gettext.h"
@@ -82,7 +81,7 @@ emalloc(size_t size)
 	fatalx_nodebug(_("internal error, tried to emalloc(0)"));
 
     if ((ptr = malloc(size)) == NULL)
-	fatalx_nodebug(NULL);
+	fatal_nodebug(NULL);
     return ptr;
 }
 
@@ -102,7 +101,7 @@ emalloc2(size_t nmemb, size_t size)
 
     size *= nmemb;
     if ((ptr = malloc(size)) == NULL)
-	fatalx_nodebug(NULL);
+	fatal_nodebug(NULL);
     return ptr;
 }
 
@@ -124,7 +123,7 @@ ecalloc(size_t nmemb, size_t size)
 	size *= nmemb;
     }
     if ((ptr = malloc(size)) == NULL)
-	fatalx_nodebug(NULL);
+	fatal_nodebug(NULL);
     memset(ptr, 0, size);
     return ptr;
 }
@@ -143,7 +142,7 @@ erealloc(void *ptr, size_t size)
 
     ptr = ptr ? realloc(ptr, size) : malloc(size);
     if (ptr == NULL)
-	fatalx_nodebug(NULL);
+	fatal_nodebug(NULL);
     return ptr;
 }
 
@@ -165,7 +164,7 @@ erealloc3(void *ptr, size_t nmemb, size_t size)
     size *= nmemb;
     ptr = ptr ? realloc(ptr, size) : malloc(size);
     if (ptr == NULL)
-	fatalx_nodebug(NULL);
+	fatal_nodebug(NULL);
     return ptr;
 }
 
@@ -189,7 +188,7 @@ erecalloc(void *ptr, size_t onmemb, size_t nmemb, size_t msize)
     size = nmemb * msize;
     ptr = ptr ? realloc(ptr, size) : malloc(size);
     if (ptr == NULL)
-	fatalx_nodebug(NULL);
+	fatal_nodebug(NULL);
     if (nmemb > onmemb) {
 	size = (nmemb - onmemb) * msize;
 	memset((char *)ptr + (onmemb * msize), 0, size);
@@ -254,7 +253,7 @@ easprintf(char **ret, const char *fmt, ...)
     va_end(ap);
 
     if (len == -1)
-	fatalx_nodebug(NULL);
+	fatal_nodebug(NULL);
     return len;
 }
 
@@ -268,6 +267,6 @@ evasprintf(char **ret, const char *format, va_list args)
     int len;
 
     if ((len = vasprintf(ret, format, args)) == -1)
-	fatalx_nodebug(NULL);
+	fatal_nodebug(NULL);
     return len;
 }
