@@ -366,7 +366,7 @@ sudo_putenv(char *str, bool dupcheck, bool overwrite)
 	if (env.envp[env.env_len] != NULL)
 	    fatalx(_("sudo_putenv: corrupted envp, length mismatch"));
 #endif
-	fatalx(NULL);
+	fatal(NULL);
     }
     debug_return_int(rval);
 }
@@ -544,21 +544,6 @@ sudo_getenv(const char *name)
     debug_return_str(val);
 }
 
-/*
- * Merge another environment with our private copy.
- */
-void
-env_merge(char * const envp[], bool overwrite)
-{
-    char * const *ep;
-    debug_decl(env_merge, SUDO_DEBUG_ENV)
-
-    for (ep = envp; *ep != NULL; ep++)
-	sudo_putenv(*ep, true, overwrite);
-
-    debug_return;
-}
-
 /*
  * Check the env_delete blacklist.
  * Returns true if the variable was found, else false.
@@ -695,6 +680,23 @@ env_should_keep(const char *var)
     debug_return_bool(keepit == true);
 }
 
+/*
+ * Merge another environment with our private copy.
+ * Only overwrite an existing variable if it is not
+ * being preserved from the user's environment.
+ */
+void
+env_merge(char * const envp[])
+{
+    char * const *ep;
+    debug_decl(env_merge, SUDO_DEBUG_ENV)
+
+    for (ep = envp; *ep != NULL; ep++)
+	sudo_putenv(*ep, true, !env_should_keep(*ep));
+
+    debug_return;
+}
+
 static void
 env_update_didvar(const char *ep, unsigned int *didvar)
 {
