@@ -61,7 +61,7 @@
 bool sudoers_setlocale(int newlocale, int *prevlocale);
 int sudoers_getlocale(void);
 void audit_success(char *exec_args[]);
-void audit_failure(char *exec_args[], char const *const fmt, ...);
+void audit_failure(char *exec_args[], char const *const fmt, ...) __printflike(2, 3);;
 void log_allowed(int status);
 void log_auth_failure(int status, int tries);
 void log_denial(int status, bool inform_user);
