@@ -228,7 +228,7 @@ unsigned long ZEXPORT crc32(crc, buf, len)
     const unsigned char FAR *buf;
     uInt len;
 {
-    if (buf == Z_NULL) return 0;
+    if (buf == Z_NULL) return 0UL;
 
 #ifdef DYNAMIC_CRC_TABLE
     if (crc_table_empty)
@@ -246,7 +246,7 @@ unsigned long ZEXPORT crc32(crc, buf, len)
             return crc32_big(crc, buf, len);
     }
 #endif /* BYFOUR */
-    crc = crc ^ (unsigned long)0xffffffff;
+    crc = crc ^ 0xffffffffUL;
     while (len >= 8) {
         DO8;
         len -= 8;
@@ -254,7 +254,7 @@ unsigned long ZEXPORT crc32(crc, buf, len)
     if (len) do {
         DO1;
     } while (--len);
-    return crc ^ (unsigned long)0xffffffff;
+    return crc ^ 0xffffffffUL;
 }
 
 #ifdef BYFOUR
@@ -389,7 +389,7 @@ local uLong crc32_combine_(crc1, crc2, len2)
         return crc1;
 
     /* put operator for one zero bit in odd */
-    odd[0] = (unsigned long)0xedb88320; /* CRC-32 polynomial */
+    odd[0] = 0xedb88320UL;          /* CRC-32 polynomial */
     row = 1;
     for (n = 1; n < GF2_DIM; n++) {
         odd[n] = row;
